package eu.coges.provaaar;

import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanResult;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.util.ArrayList;

import eu.coges.btdatakeymodule.BTDataKeyModule;
import eu.coges.btdatakeymodule.BTDataKeyModuleIface;

/*import com.orm.query.Select;*/

public class ScanDevicesActivity extends AppCompatActivity implements BTDataKeyModuleIface {

    public static final String EXTRAS_DEVICE_NAME = "DEVICE_NAME";
    public static final String EXTRAS_DEVICE_ADDRESS = "DEVICE_ADDRESS";

    public static final String EXTRAS_INTENT_ACTION = "INTENT_ACTION";
    public static final String EXTRAS_INTENT_SUBACTION = "INTENT_SUBACTION";
    public static final String EXTRAS_DEVICECONTROL_ACTION = "Device Control";

    public static final String EXTRAS_OPCODE_ACTION = "Set Operator code";
    public static final String EXTRAS_GETDATA_ACTION = "Get Data";

    private BluetoothAdapter mBluetoothAdapter;
    private BluetoothLeScanner mBluetoothLeScanner = null;
    private ScanCallback mScanCallback = null;
    private boolean mScanning;
    private Handler mHandler;

    private String mDevAddr = null;
    private boolean mDevAddrFound = false;
    private boolean mStarted = false;

    // TO TEST BTDataKeyModule
    private BluetoothDevice mDevice;
    private boolean isJustInited = true;
    private BTDataKeyModule mBTDataKeyModule = new BTDataKeyModule();
    // END of TEST BTDataKeyModule

    // These are to manage the RecyclerView
    protected RecyclerView mRecyclerView;
    protected LeDeviceListAdapter mLeDeviceListAdapter;    // Custom Adapter, that provides data to RecyclerView
    protected RecyclerView.LayoutManager mLayoutManager;

    private int mMessageCounter = 0;

    private void showToast(final String message, final int length) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast t = Toast.makeText(ScanDevicesActivity.this, message, length);
                t.show();
            }
        });
    }

    private void showToast(final String message) {
        showToast(message, Toast.LENGTH_LONG);
    }

    @Override
    public void onBTDataKeyIDdataReceived(byte[] raw, String serialNum, String hwId, String hwVer, String productionDate, String fwVersion) {

        final String message = "onBTDataKeyIDdataReceived:\n Serial: " + serialNum +
                "\n HW ID: " + hwId +
                "\n Ver: " + hwVer +
                "\n Manuf. on: " + productionDate +
                "\n FW Ver: BTDataKey v" + fwVersion;
        showToast(message, Toast.LENGTH_LONG);

/*
        showToast(message, Toast.LENGTH_LONG);
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                AlertDialog.Builder builder = new AlertDialog.Builder(ScanDevicesActivity.this);
                builder.setMessage(message)
                        .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }})
                        .setCancelable(true);
                builder.create().show();
            }
        });
*/


        mBTDataKeyModule.getBTDataKeyBattLevel();
    }

    @Override
    public void onBTDataKeyPendingStatusReceived(boolean isDataPending) {
        showToast("onBTDataKeyPendingStatusReceived - \n" + (isDataPending ? "Data is PENDING": "Key is EMPTY"));
    }

    @Override
    public void onBTDataKeyDataPacketReceived(int messageCounter, String message) {
        if ((messageCounter == 0) || (messageCounter > mMessageCounter + 9)) {
            showToast("Data Pck Rcvd - #" + messageCounter, Toast.LENGTH_SHORT);
            mMessageCounter = messageCounter;
        }
    }

    @Override
    public void onBTDataKeyBatteryLevelReceived(int battLevel) {
        showToast("onBTDataKeyBatteryLevelReceived: " + battLevel, Toast.LENGTH_SHORT);
        /*mBTDataKeyModule.setBTDataKeyOperatorCode("00000000","00000000");*/

        /* Uncomment the line below in order to check the pending status of the Key */
        //mBTDataKeyModule.getBTDataKeyPendingStatus();

        mMessageCounter = 0;    // Reset data message counter
        byte passwordIRDA[] = null; /*new byte[]{0x11,0x11,0x11,0x11};*/
        mBTDataKeyModule.startDataRecording(passwordIRDA);

        /* Uncomment the line below in order to delete data from the Key */
        //mBTDataKeyModule.startDeleteData();
    }

    @Override
    public void onBTDataKeyOpCodeConfigurationSuccess() {
        showToast("onBTDataKeyOpCodeConfigurationSuccess");

        // Provide the Password IRDA or null for default ("00000000")
        /*byte passwordIRDA[] = new byte[]{1,1,1,1,1,1,1,1};
        mBTDataKeyModule.startDataRecording(passwordIRDA);*/
    }

    @Override
    public void onBTDataKeyAuditReceived(byte[] raw, String dataKeyMode, String auditText, final String auditPath) {
        String str = "";
        try {
            str = new String(raw, "UTF-8");
        } catch (Exception e) {

        }

        final AlertDialog.Builder builder = new AlertDialog.Builder(this);

        builder.setMessage("Eva-Dts file succesfully received.\n\nIt has also been saved in:\n\n" + auditPath)
                .setCancelable(false)
                .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        File file = new File(auditPath);
                        boolean deleted = file.delete();
                        showToast("Cancellato " + deleted);
                        onBackPressed();
                    }
                });

        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                AlertDialog dialog = builder.create();
                dialog.show();
            }
        });
    }

    @Override
    public void onBTDataKeyDeleteSuccess() {
        showToast("onBTDataKeyDeleteSuccess");
    }

    @Override
    public void onBTDataKeyError(int errorCode) {

        String errMsg;
        switch (errorCode) {
            case BTDataKeyModule.ERR_BTDK_NONE:
                return;
            case BTDataKeyModule.ERR_BTDK_ILLEGAL_STATUS:
                errMsg = "ERR_BTDK_ILLEGAL_STATE";
                break;
            case BTDataKeyModule.ERR_BTDK_NOT_RESPONDING:
                errMsg = "ERR_BTDK_NOT_RESPONDING";
                break;
            case BTDataKeyModule.ERR_BTDK_DATA_REC_TIMEOUT:
                errMsg = "ERR_BTDK_DATA_REC_TIMEOUT";
                break;
            case BTDataKeyModule.ERR_BTDK_UNLOCKED:
                errMsg = "ERR_BTDK_UNLOCKED";
                break;
            case BTDataKeyModule.ERR_BTDK_OPCODE_SIZE:
                errMsg = "ERR_BTDK_OPCODE_SIZE";
                break;
            case BTDataKeyModule.ERR_BTDK_OPCODE_FORMAT:
                errMsg = "ERR_BTDK_OPCODE_FORMAT";
                break;
            case BTDataKeyModule.ERR_BTDK_OPCODE_UNLOCK_FAIL:
                errMsg = "ERR_BTDK_OPCODE_UNLOCK_FAIL";
                break;
            case BTDataKeyModule.ERR_BTDK_OPCODE:
                errMsg = "ERR_BTDK_OPCODE";
                break;
            case BTDataKeyModule.ERR_BTDK_IRCODE:
                errMsg = "ERR_BTDK_IRCODE";
                break;
            default:
                errMsg = String.format("Error Code [%d]",errorCode);
        }
        showToast("onBTDataKeyError: " + errMsg);
    }

    @Override
    public void onBTDataKeyModuleError(int errorCode) {
        String errMsg;
        switch (errorCode) {
            case BTDataKeyModule.ERR_NONE:
                return;
            case BTDataKeyModule.ERR_BLE_CONNECTION:
                errMsg = "ERR_BLE_CONNECTION";
                break;
            case BTDataKeyModule.ERR_EVA_DATA_WRITE:
                errMsg = "ERR_EVA_DATA_WRITE";
                break;
            case BTDataKeyModule.ERR_INTERRUPTED_PROCESS:
                errMsg = "ERR_INTERRUPTED_PROCESS";
                break;
            default:
                errMsg = String.format("Error Code [%d]",errorCode);
        }
        showToast("onBTDataKeyModuleError: " + errorCode);
    }

    @Override
    public void onBTDataKeyStatusChange(int statusCode) {
        switch (statusCode) {
            case BTDataKeyModule.S_DISCONNECTED:
                if (isJustInited) {
                    this.mBTDataKeyModule.connect(mDevice);
                    isJustInited = false;
                }
                else {
                    showToast("BTDataKey has disconnected!");
                }
                break;
            case BTDataKeyModule.S_READY:
            {
                // This start the DATA recording on BTDataKey
                //  * After successful transmission of Eva-Dts file from BTDataKey to App
                //  * onBTDataKeyAuditReceived() will be called, otherwise either
                //  * onBTDataKeyErr or onBTDataKeyModuleError will be called
                //mBTDataKeyModule.startDataRecording();
                mBTDataKeyModule.getBTDataKeyIDdata();
            }
            break;
        }
    }

    // Stops scanning after 10 seconds.
    private static final long SCAN_PERIOD       = 10000;
    private static final long SHORT_SCAN_PERIOD = 5000;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_scan_devices);

        ActionBar actionbar = getSupportActionBar();

        if (actionbar != null) {
            actionbar.setTitle(R.string.title_scan_devices);
            actionbar.setDisplayHomeAsUpEnabled(true);
        }

        mHandler = new Handler();
        mStarted = false;

        // Use this check to determine whether BLE is supported on the device.  Then you can
        // selectively disable BLE-related features.
        if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
            Toast.makeText(this, R.string.ble_not_supported, Toast.LENGTH_SHORT).show();
            finish();
        }

        // Initializes a Bluetooth adapter.  For API level 18 and above, get a reference to
        // BluetoothAdapter through BluetoothManager.
        final BluetoothManager bluetoothManager =
                (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();

        // Checks if Bluetooth is supported on the device.
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, R.string.error_bluetooth_not_supported, Toast.LENGTH_SHORT).show();
            finish();
            return;
        }

        if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            mBluetoothLeScanner = mBluetoothAdapter.getBluetoothLeScanner();
            mScanCallback = new ScanCallback() {
                @Override
                public void onScanResult(int callbackType, final ScanResult result) {
                    super.onScanResult(callbackType, result);

                    /*
                    * On Scan Result: we populate the list of devices we can connect to. If the device
                    * is found in the keyRegistry the device is simply added to the list, otherwise it
                    * is added but the layout of the list item is "greyed".
                    *
                    * If a favorite device is found, it is used for further processing
                    * */

                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                        /*Toast t = Toast.makeText(getApplicationContext(), "Identified BLE: " + device
                                                    + "\n with rssi: " + String.valueOf(rxrssi), Toast.LENGTH_LONG);
                        t.show();*/
                            if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                                //if (mDevAddr == null) {
                                String devName = result.getDevice().getName();
                                String devAddr = result.getDevice().getAddress();

                                if (devName != null && (devName.length() < 5 || devName.subSequence(0, 5).equals("BTCOG"))) {
                                    mLeDeviceListAdapter.addDevice(result.getDevice(), false, false);
                                }

                            }
                        }
                    });
                }
            };
        }

        mRecyclerView = (RecyclerView)findViewById(R.id.recyclerview);
        /*if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            mRecyclerView.addItemDecoration(new DividerItemDecoration(ScanDevicesActivity.this.getDrawable(android.R.drawable.divider_horizontal_bright)));
        } else {
            mRecyclerView.addItemDecoration(new DividerItemDecoration(getResources().getDrawable(android.R.drawable.divider_horizontal_bright)));
        }*/

        mLayoutManager = new LinearLayoutManager(this);
        mRecyclerView.setLayoutManager(mLayoutManager);
        mLeDeviceListAdapter = new LeDeviceListAdapter();
        mRecyclerView.setAdapter(mLeDeviceListAdapter);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.ble_scan_main, menu);
        if (!mScanning) {
            menu.findItem(R.id.menu_stop).setVisible(false);
            menu.findItem(R.id.menu_scan).setVisible(true);
            menu.findItem(R.id.menu_refresh).setActionView(null);
        } else {
            menu.findItem(R.id.menu_stop).setVisible(true);
            menu.findItem(R.id.menu_scan).setVisible(false);
            menu.findItem(R.id.menu_refresh).setActionView(
                    R.layout.actionbar_indeterminate_progress);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_scan:
                mLeDeviceListAdapter.clear();
                scanLeDevice(true);
                break;
            case R.id.menu_stop:
                scanLeDevice(false);
                break;
            case android.R.id.home:
                scanLeDevice(false);
                startActivity(new Intent(getApplicationContext(),
                        MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP));
                return true;
        }
        return true;
    }

    @Override
    protected void onResume() {
        super.onResume();

        // Initializes list view adapter.
        mLeDeviceListAdapter = new LeDeviceListAdapter();
        mRecyclerView.setAdapter(mLeDeviceListAdapter);

        mDevAddr = null;
        mDevAddrFound = false;
        mStarted = true;

        // Start Scan of BLE Devices
        scanLeDevice(true);
    }

    @Override
    protected void onPause() {
        super.onPause();

        // Stops Scan of BLE Devices
        scanLeDevice(false);
        mLeDeviceListAdapter.clear();

        mBTDataKeyModule.deinit();
    }

    private void scanLeDevice(final boolean enable) {
        if (enable) {
            // Stops scanning after a pre-defined scan period.
            long scan_period = SCAN_PERIOD;
            if (mDevAddr != null) {
                scan_period = SHORT_SCAN_PERIOD;
            }
            mHandler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    if (mDevAddr != null && !mDevAddrFound) {
                        // Return an AlertDialog that informs that the default BLE device has not been found... continue scanning
                        mDevAddr = null;

                        AlertDialog.Builder builder = new AlertDialog.Builder(ScanDevicesActivity.this);
                        builder.setMessage(R.string.ble_not_found_dialog_message)
                                .setTitle(R.string.ble_not_found_dialog_title)
                                .setCancelable(false)
                                .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        mHandler.postDelayed(new Runnable() {
                                            @Override
                                            public void run() {
                                                mScanning = false;
                                                if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                                                    mBluetoothLeScanner.stopScan(mScanCallback);
                                                } else {
                                                    mBluetoothAdapter.stopLeScan(mLeScanCallback);
                                                }
                                                invalidateOptionsMenu();
                                            }
                                        }, SCAN_PERIOD);
                                    }
                                });
                        AlertDialog alert = builder.create();
                        alert.show();
                    } else {
                        mScanning = false;
                        if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                            mBluetoothLeScanner.stopScan(mScanCallback);
                        } else {
                            mBluetoothAdapter.stopLeScan(mLeScanCallback);
                        }
                        invalidateOptionsMenu();
                    }
                }
            }, scan_period);

            mScanning = true;
            if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                //ArrayList<ScanFilter> scanFiltList = new ArrayList<>();
                //scanFiltList.add(new ScanFilter.Builder().setServiceUuid(new ParcelUuid(BluetoothLeService.UUID_COGES_SERVICE)).build());
                //ScanSettings scanSettings = new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build();
                //mBluetoothLeScanner.startScan(scanFiltList, scanSettings, mScanCallback);
                mBluetoothLeScanner.startScan(mScanCallback);
            } else {
                mBluetoothAdapter.startLeScan(mLeScanCallback);
            }
        } else {
            mScanning = false;
            if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                mBluetoothLeScanner.stopScan(mScanCallback);
            } else {
                mBluetoothAdapter.stopLeScan(mLeScanCallback);
            }
        }
        invalidateOptionsMenu();
    }

    // TODO - Move this within the module BTDataKey Module Init
    /*@Override
    public void onDestroy() {
        stopService(new Intent(this, BluetoothLeService.class));
        super.onDestroy();
    }*/

    private void sendIntentToTargetActivity(BluetoothDevice device) {
        if (mScanning) {
            if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                mBluetoothLeScanner.stopScan(mScanCallback);
            } else {
                mBluetoothAdapter.stopLeScan(mLeScanCallback);
            }
            mScanning = false;
        }
        /*startActivity(intent);*/
        isJustInited = true;
        mDevice = device;
        mBTDataKeyModule.init(this, ScanDevicesActivity.this);
        /*mBTDataKeyModule.setListener(ScanDevicesActivity.this);*/
    }

    public void onItemClick(View v, int position) {
        final BluetoothDevice device = mLeDeviceListAdapter.getDevice(position);
        if (device == null) return;

        // Send Intent with the selected BluetoothDevice to DeviceControl or Opcode Activity
        mHandler.postDelayed(new Runnable() {
            @Override
            public void run() {
                sendIntentToTargetActivity(device);
            }
        }, 200);
    }

    // Adapter for holding devices found through scanning.
    private class LeDeviceListAdapter extends RecyclerView.Adapter<ViewHolder> {

        private ArrayList<BluetoothDevice> mLeDevices;

        Drawable key;

        LeDeviceListAdapter() {
            mLeDevices = new ArrayList<>();

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                key = getResources().getDrawable(R.drawable.ic_launcher_coges_alpha, null);
            } else {
                key = getResources().getDrawable(R.drawable.ic_launcher_coges_alpha);
            }
        }

        @Override
        public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
            View v = LayoutInflater.from(viewGroup.getContext())
                    .inflate(R.layout.list_bleitem_device, viewGroup, false);

            return new ViewHolder(v);
        }

        @Override
        public void onBindViewHolder(ViewHolder viewHolder, int i) {
            BluetoothDevice item = mLeDevices.get(i);
            viewHolder.deviceName.setText(item.getName());
            viewHolder.deviceAddress.setText(item.getAddress());
            viewHolder.iconImage.setImageDrawable(key);
        }

        @Override
        public int getItemCount() {
            return mLeDevices.size();
        }

        public void addDevice(BluetoothDevice device, Boolean isKnown, Boolean isFavorite) {
            if(!mLeDevices.contains(device)) {
                mLeDevices.add(device);
                notifyDataSetChanged();
            }
        }

        public BluetoothDevice getDevice(int position) {
            return mLeDevices.get(position);
        }

        public void clear() {
            mLeDevices.clear();
            notifyDataSetChanged();
            notifyItemRangeChanged(0,0);
        }
    }

    private class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener /*, AlertDialog.OnClickListener*/{

        private TextView deviceName;
        private TextView deviceAddress;
        private ImageView iconImage;
        private ViewGroup device_item;

        /*private final AlertDialog mDialog;*/

        public ViewHolder(View v) {
            super(v);

            deviceName = (TextView)v.findViewById(R.id.device_name);
            deviceAddress = (TextView)v.findViewById(R.id.device_address);
            iconImage = (ImageView)v.findViewById(R.id.imageViewDeviceIcon);

            device_item = (ViewGroup)v.findViewById(R.id.ble_list_item);
            device_item.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {

            if (v.equals(this.device_item)) {
                ScanDevicesActivity.this.onItemClick(v,getAdapterPosition());
            }
        }
    }

    // Device scan callback.
    private BluetoothAdapter.LeScanCallback mLeScanCallback =
        new BluetoothAdapter.LeScanCallback() {

            @Override
            public void onLeScan(final BluetoothDevice device, int rssi, byte[] scanRecord) {

                //final int rxrssi = rssi;

                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        String devName = device.getName();

                        if (devName != null && (devName.length() < 5 || devName.subSequence(0, 5).equals("BTCOG"))) {
                            mLeDeviceListAdapter.addDevice(device, false, false);
                        }
                    }
                });
            }
        };
}
